import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';


export default class LineChartComponent extends Component {
  @tracked lineElement;

  get isLoaded() {
    return (this.args.dataset || this.args.dataset === []) && !!this.args.dimensions;
  }

  get fill() {
    return this.args.fill ? this.args.fill : 'none'
  }

  get fillOpacity() {
    return this.args.fillOpacity || '0.5'
  }

  get color() {
    return this.args.color ? this.args.color : 'black'
  }

  get strokeWidth() {
    return this.args.strokeWidth ? this.args.strokeWidth : '1'
  }

  get xKey() {
    return this.args.xKey ? this.args.xKey : 'OCCURED';
  }

  get yKey() {
    return this.args.yKey ? this.args.yKey : 'SUM';
  }

  get fillIn() {
    return this.fill !== 'none';
  }

  get showNoData() {
    return this.args.showNoData || false;
  }

  // Options: 'left', 'middle'
  // This shifts the datapoints to be center aligned with bar charts if set to 'middle' for overlay with barchart
  get xValueAnchor() {
    return this.args.xValueAnchor || 'left';
  }

  get middleAnchorOffset() {
    if (!this.args.dataset || this.args.dataset.length < 2) return 0;
    const xScale = this.args.xScale;
    const dateOne = this.getDateValue(this.args.dataset[1]);
    const dateTwo = this.getDateValue(this.args.dataset[0]);
    return Math.abs(xScale(dateOne) - xScale(dateTwo))/2;
  }

  // If a filled in line graph (aka svg polygon) where do we draw the floor?
  get graphBottom() {
    return this.args.graphBottom || -10;
  }

  get currentLineInfo() {
    if (!this.isLoaded) return {}
    if (this.args.dataset.length === 0 && this.showNoData) return this.noDataLine();

    return this.fillIn
      ? this.generateSVGPolygon(this.args.yScale, this.args.dataset)
      : this.generateSVGLine(this.args.yScale, this.args.dataset)
  }

  noDataLine() {
    const yPosition = this.args.dimensions.height - this.strokeWidth;

    return `M0,${yPosition} L${this.args.dimensions.width},${yPosition} `
  }

  generateSVGLine(yScale, dataset) {
    if ( !dataset.length ) return ''
    const yKey = this.yKey;
    const firstY = dataset[0] ? dataset[0][yKey] : 0;
    return `M${this.findXPosition(dataset[0])} ${yScale(firstY)} 
            ${dataset.slice(1).map(d => {
              return `L${this.findXPosition(d)} ${yScale(d[yKey])}`;
            }).join(' ')}`
  }

  // Polygon svg shape if fill is specified and we want area under line to be filled in with fill color
  generateSVGPolygon(yScale, dataset) {
    const {xKey, yKey} = this;
    const points = this.addPolygonDatapoints(dataset);
    return points.map((point) => {
      if (point[xKey] == null) return ""
      return point[yKey] || point[yKey] === 0 ? `${this.findXPosition(point)},${yScale(point[yKey])}` : "";
    }).join(' ');
  }

  addPolygonDatapoints(dataset) {
    if(!dataset || dataset.length === 0) return [];
    const bottomLeftCorner = {};
    const bottomRightCorner = {};
    bottomLeftCorner[this.xKey] = dataset[0][this.xKey];
    bottomLeftCorner[this.yKey] = this.graphBottom;

    bottomRightCorner[this.xKey] = dataset[dataset.length - 1][this.xKey]
    bottomRightCorner[this.yKey] = this.graphBottom;

    return [
      bottomLeftCorner,
      ...dataset,
      bottomRightCorner
    ]
  }
  
  findXPosition(datapoint) {
    const xScale = this.args.xScale;
    const date = this.getDateValue(datapoint);
    if (this.xValueAnchor === 'middle') {
      return xScale(date) + this.middleAnchorOffset;
    } else {
      return xScale(date);
    }
  }

  getDateValue(datapoint) {
    if (!datapoint) return;
    const dateVal = datapoint[this.xKey];
    return typeof dateVal === 'string' ? new Date(dateVal) : dateVal; 
  }
}