import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';
import moment from 'moment';
import { bisector } from "d3-array";
import { throttle } from '@ember/runloop';

export default class ScrubberComponent extends Component {
  get bisectDate() {
    return bisector(d => d.at).left;
  }

  @tracked fadeState = "opacity-0";
  @tracked currentMousePos;
  @tracked lineTextValue;
  @tracked cornerTextValue;
  @tracked line;
  @tracked movingText;

  get strokeWidth() {
    return this.args.width ? this.args.width : '3';
  }

  get color() {
    return this.args.color ? this.args.color : 'gray';
  }

  get lineHeight() {
    return this.args.chart.getBoundingClientRect().height;
  }

  get lineTextUnits() {
    return this.args.lineTextUnits ? this.args.lineTextUnits : '';
  }

  get cornerTextUnits() {
    return this.args.cornerTextUnits ? this.args.cornerTextUnits : '';
  }

  get lineTextKey() {
    return this.args.lineTextKey ? this.args.lineTextKey : 'at'
  }

  get cornerTextKey() {
    return this.args.cornerTextKey ? this.args.cornerTextKey : 'value'
  }

  get lineText() {
    if(this.lineTextValue instanceof Date) {
      return this.formatDate(this.lineTextValue);
    } else {
      return `${this.lineTextValue}${this.lineTextUnits}`;
    }
  }

  get cornerText() {
    if(this.cornerTextValue instanceof Date) {
      return this.formatDate(this.cornerTextValue);
    } else {
      return `${this.cornerTextValue}${this.cornerTextUnits}`;
    }
  }

  updateLine = (e) => {
    const mousePos = this.mouseOrTouchPosition(e);
    if(this.args.dataset.length && mousePos > 0.5) {
      const d = this.nearestDataPoint(mousePos);
      if (!isNaN(mousePos)) {
        this.currentMousePos = mousePos;
        this.lineTextValue = d[this.lineTextKey];
        this.cornerTextValue = d[this.cornerTextKey];
      }
    }
  }

  nearestDataPoint(mousePos) {
    const x0 = this.args.xScale.invert(mousePos); // x position within the element.
    const i = this.bisectDate(this.args.dataset, x0, 1);
    const d0 = this.args.dataset[i - 1];
    const d1 = this.args.dataset[i];
    
    return (d1 && d0) ? (x0 - d0.at > d1.at - x0 ? d1 : d0) : 0;
  }

  mouseOrTouchPosition(event) {
    const clientX = event.type.includes("touch") ? event.targetTouches[0].clientX : event.clientX;
    return clientX - this.args.chart.getBoundingClientRect().left;
  }

  formatDate(date) {
    const momentDate = moment(date);
    const currentYear = momentDate.year() === moment().year();

    return moment(date).format(`MMM D ${currentYear ? '' : 'YYYY'}, h:mm a`);
  }

  mousemove = (e) => {
    throttle(this, this.updateLine, e, 20);
  }

  mouseout = () => {
    this.fadeState = "opacity-0";
  }

  mouseover = (e) => {
    if(e.type === "touchstart") {
      this.currentMousePos = this.mouseOrTouchPosition(e);
      this.setMovingElementsPosition(this.currentMousePos);
    }
    this.fadeState = "opacity-100";
  }

  initElement = (element, [elementName]) => {
    this[elementName] = element;
  }

  animateScrubbing = () => {
    const x = (this.currentMousePos || this.currentMousePos === 0)
            ? this.getAnimatedLocation(this.line, 'x1')
            : 0

    if(x) {
      this.setMovingElementsPosition(x);
    }
  }

  setMovingElementsPosition = (x) => {
    this.line.setAttribute('x1', x)
    this.line.setAttribute('x2', x)
    this.movingText.setAttribute('x', x)
  }

  getAnimatedLocation(element, attrName) {
    const currentX = parseInt(element.getAttribute(attrName)); // Get current position
    const totalDx = this.currentMousePos - currentX; // Find total x distance from cursor to element
    const acceleration = 4; // acceleration scalar
    const frameDx = totalDx / acceleration; // delta x for this frame

    return Math.abs(frameDx) > 1                      // if the delta x is less than 1 it needs to be
            ? currentX + frameDx                      // rounded or else it will register as 0px and get stuck
            : totalDx < 1 ? this.currentMousePos : currentX + 1; // note only happens really close to cursor as it deccelerates
  }
}